@@ -1,4 +1,4 @@
-# lib.R: miscellaneous functions for both earth and plotmo
+# lib.R: miscellaneous functions for plotmo and related packages
 #        functions in this file are in alphabetical order
 
 any1 <- function(x)
@@ -75,7 +75,7 @@ check.classname <- function(object, substituted.object, allowed.classnames)
 {
     err.msg <- quotify(allowed.classnames)
     if(length(allowed.classnames) > 1)
-        err.msg <- sprintf("one of\n%s", err.msg)
+        err.msg <- sprint("one of\n%s", err.msg)
     if(is.null(object))
         stopf("object is NULL but expected an object of class of %s",
               err.msg)
@@ -130,7 +130,7 @@ check.no.na.in.mat <- function(object)
                 if(!is.null(colnames(object)))
                     colnames(object)[icol]
                 else
-                    sprintf("%s[,%d]",
+                    sprint("%s[,%d]",
                         short.deparse(substitute(object), "matrix"), icol)
 
             check(object, check.name, "NA", is.na, na.ok=FALSE)
@@ -203,13 +203,13 @@ check.that.most.are.positive <- function(x, xname, user.arg, non.positive.msg, f
     nonpos <- x <= 0
     if(sum(nonpos, na.rm=TRUE) > frac.allowed * length(x)) { # more than frac.allowed nonpos?
         ifirst <- which(nonpos)[1]
-        stop0(sprintf(
+        stop0(sprint(
                 "%s is not allowed because too many %ss are %s\n",
                 user.arg, unquote(xname), non.positive.msg),
-              sprintf(
+              sprint(
                 "       %.2g%% are %s (%g%% is allowed)\n",
                 100 * sum(nonpos) / length(x), non.positive.msg, 100 * frac.allowed),
-               sprintf("       e.g. %s[%d] is %g", unquote(xname), ifirst, x[ifirst]))
+               sprint("       e.g. %s[%d] is %g", unquote(xname), ifirst, x[ifirst]))
     }
 }
 check.vec <- function(object, object.name, expected.len=NA, logical.ok=TRUE, na.ok=FALSE)
@@ -237,9 +237,9 @@ colname <- function(object, i, object.name=trunc.deparse(substitute(object)))
     if(!is.null(colnames))
         colnames[i]
     else if(NCOL(object) > 1)
-        sprintf("%s[,%g]", object.name, i)
+        sprint("%s[,%g]", object.name, i)
     else
-        sprintf(object.name)
+        sprint(object.name)
 }
 # if trace>0 or the func fails, then print the call to func
 
@@ -382,8 +382,14 @@ get.model.env <- function(object, object.name="object", trace=0)
     if(!is.try.err(terms) && !is.null(terms)) {
         model.env <- attr(terms, ".Environment")
         if(is.null(model.env)) {
-            if(inherits(object, "glmnet.formula")) # glmnetUtils package
-                stop0("glmnet.formula must be called with use.model.frame=TRUE")
+            if(inherits(object, "glmnet.formula") ||  # glmnetUtils package
+               inherits(object, "cv.glmnet.formula"))
+                if(inherits(object, "glmnet.formula"))
+                    stop0(
+"for this plot, glmnet.formula must be called with use.model.frame=TRUE")
+                if(inherits(object, "cv.glmnet.formula"))
+                    stop0(
+"for this plot, cv.glmnet.formula must be called with use.model.frame=TRUE")
             stop0("attr(terms, \".Environment\") is NULL")
         }
         if(!is.environment(model.env))
@@ -467,7 +473,7 @@ imatch.choices <- function(arg, choices,
                if(err.msg.has.index) " an integer index or" else "",
                if(nchar(err.msg.ext)) paste0(" ", err.msg.ext, " or") else "")
     if(nchar(err.msg) == 0)
-        err.msg <- sprintf("Choose%s one of: %s", err.msg.ext, quotify(choices))
+        err.msg <- sprint("Choose%s one of: %s", err.msg.ext, quotify(choices))
     if(!is.character(arg) || length(arg) != 1 || !nzchar(arg))
          stopf("illegal '%s' argument\n%s", argname, err.msg)
     imatch <- pmatch(arg, choices)
@@ -646,14 +652,14 @@ paste.trunc <- function(..., sep=" ", collapse=" ", maxlen=60)
 }
 pastef <- function(s, fmt, ...) # paste the printf style args to s
 {
-    paste0(s, sprintf(fmt, ...))
+    paste0(s, sprint(fmt, ...))
 }
 print.first.few.elements.of.vector <- function(x, trace, name=NULL)
 {
     try(cat(" min", min(x), "max", max(x)), silent=TRUE)
     spaces <- "               "
     if(!is.null(name))
-        spaces <- sprintf("%*s", nchar(name), " ")  # nchar spaces
+        spaces <- sprint("%*s", nchar(name), " ")  # nchar spaces
     cat0("\n", spaces, " value")
     len <- if(trace >= 4) length(x)
            else           min(if(is.logical(x)) 20 else 10, length(x))
@@ -671,16 +677,34 @@ print.first.few.elements.of.vector <- function(x, trace, name=NULL)
         print(summary(x))
     }
 }
+# A safe version of sprintf.
+# Like sprintf except that %s on NULL prints "NULL" rather than
+# preventing the entire string from being printed
+#
+# e.g. sprintf("abc %s def", NULL) returns an empty string -- a silent failure!
+# but   sprint("abc %s def", NULL) returns "abc NULL def"
+#
+# e.g. sprintf("abc %d def", NULL) returns an empty string!
+# but   sprint("abc %d def", NULL) causes an error msg (not a silent failure)
+
+sprint <- function(fmt, ...)
+{
+    dots <- list(...)
+    for(i in seq_along(dots))
+        if(is.null(dots[[i]]))
+            dots[[i]] <- "NULL"
+    do.call(sprintf, c(fmt, dots))
+}
 printf <- function(fmt, ...) # like c printf
 {
-    cat(sprintf(fmt, ...), sep="")
+    cat(sprint(fmt, ...), sep="")
 }
 # like printf but wrap at terminal width
 # exdent=NULL for automatic determination of xdent (line up to func opening paren)
 # TODO maxlen seems to be ignored, strwrap truncates before that?
 printf.wrap <- function(fmt, ..., exdent=NULL, maxlen=2000)
 {
-    s <- paste.trunc(paste.collapse(sprintf(fmt, ...)), maxlen=maxlen)
+    s <- paste.trunc(paste.collapse(sprint(fmt, ...)), maxlen=maxlen)
     if(is.null(exdent)) {
         # align to opening paren of func call e.g. "graphics::par(xxx)" or "foo$method("
         # TODO this doesn't account for leading newlines if any
@@ -725,9 +749,9 @@ quote.deparse <- function(object, alternative="object")
 quote.with.c <- function(names) # return "x" or c("x1", "x2")
 {
     if(length(names) == 1)
-        sprintf("\"%s\"", names)
+        sprint("\"%s\"", names)
     else
-        sprintf("c(%s)", paste0("\"", paste(names, collapse="\", \""), "\""))
+        sprint("c(%s)", paste0("\"", paste(names, collapse="\", \""), "\""))
 }
 quotify <- function(s, quote="\"") # add quotes and collapse to a single string
 {                                  # called quotify because quote is taken
@@ -817,7 +841,7 @@ stop0 <- function(...)
 }
 stopf <- function(fmt, ...) # args like printf
 {
-    stop(sprintf(fmt, ...), call.=FALSE)
+    stop(sprint(fmt, ...), call.=FALSE)
 }
 # stop if s is not a one element character vector
 stopifnot.string <- function(s, name=short.deparse(substitute(s)),
@@ -902,13 +926,13 @@ trace1 <- function(trace, fmt, ...)
 {
     stopifnot(is.numeric(trace))
     if(trace >= 1)
-        cat(sprintf(fmt, ...), sep="")
+        cat(sprint(fmt, ...), sep="")
 }
 trace2 <- function(trace, fmt, ...)
 {
     stopifnot(is.numeric(trace))
     if(trace >= 2)
-        cat(sprintf(fmt, ...), sep="")
+        cat(sprint(fmt, ...), sep="")
 }
 # Truncate deparse(object) if it is too long.
 # Necessary because deparse(substitute(x)) might return something very
@@ -967,7 +991,7 @@ warn.if.not.all.finite <- function(object, text="unknown")
 }
 warnf <- function(fmt, ...) # args like printf
 {
-    warning(sprintf(fmt, ...), call.=FALSE)
+    warning(sprint(fmt, ...), call.=FALSE)
 }
 warning0 <- function(...)
 {
