@@ -1,28 +1,113 @@
-# linmod.R: example S3 linear model
+# linmod.R: Example S3 linear model.
 #
-# Please see www.milbo.org/doc/modguide.pdf.
-#
-# This code can be used as a template for new models as follows:
-# (i)  replace all occurrences of "linmod" with the new model name,
-# (ii) modify linmod.fit and the method functions (predict.linmod and friends).
-#
-# The subset, weights and na.action arguments aren't supported.
-# Multiple responses aren't supported (y must be a vector or have one column).
-#
-# Comprehensive tests for this code are in test.linmod.R in the plotmo package.
+# See www.milbo.org/doc/modguide.pdf.
+# This software may be freely used.
 
 linmod <- function(...) UseMethod("linmod")
 
-# internal function, not for the casual user
-# for model with an intercept, first column of x must be the intercept (all 1s)
-
+linmod.default <- function(x = stop("no 'x' argument"),
+                           y = stop("no 'y' argument"),
+                           keep = FALSE,
+                           ...)
+{
+    stop.if.dot.arg.used(...)
+    xmat <- as.matrix(x)
+    # use name "(Intercept)" here so coef names match linmod.formula
+    x <- cbind("(Intercept)" = 1, xmat)
+    fit <- linmod.fit(x, y)
+    fit$call <- match.call()
+    if(keep) {
+        fit$x <- xmat
+        # save y as a matrix so can use colname to save response name
+        colname <- deparse(substitute(y))[1]
+        colname <- gsub(" ", "", substr(colname, 1, 100)) # strip spaces, truncate
+        fit$y <- as.matrix(y, ncol = 1)
+        colnames(fit$y) <- colname
+    }
+    fit
+}
+linmod.formula <- function(formula = stop("no 'formula' argument"),
+                           data = parent.frame(),
+                           keep = FALSE,
+                           ...)
+{
+    stop.if.dot.arg.used(...)
+    if(is.matrix(data))             # allow data to be a matrix
+        data <- as.data.frame(data) # will create colnames V1 V2 V3 if necessary
+    # note that na.action=na.pass because we will catch NAs later
+    # in linmod.fit, for uniformity with linmod.default
+    mf <- model.frame(formula = formula, data = data, na.action = na.pass)
+    terms <- attr(mf, "terms")
+    x <- model.matrix(terms, mf)
+    y <- model.response(mf)
+    fit <- linmod.fit(x, y)
+    fit$call <- match.call()
+    fit$terms <- terms
+    fit$xlevels <- .getXlevels(terms, mf) # for use by predict.linmod
+    if(keep)
+        fit$data <- data
+    fit
+}
 linmod.fit <- function(x = stop("no 'x' argument"),
                        y = stop("no 'y' argument"),
                        ...)
 {
+    # internal function, not for the casual user
+    # if model has an intercept, the first col of x must be intercept (all 1s)
+
     stop.if.dot.arg.used(...)
     x <- check.linmod.x(x)
     y <- check.linmod.y(x, y)
+    fit <- do.linmod.fit(x, y)
+    class(fit) <- "linmod"
+    fit
+}
+check.linmod.x <- function(x)
+{
+    if(!is.matrix(x))
+        stop("'x' is not a matrix or could not be converted to a matrix")
+    if(NROW(x) == 0 || NCOL(x) == 0)
+        stop("'x' is empty")
+    if(anyNA(x))
+        stop("NA in 'x'")
+    # checking just the first column of x suffices because x is a matrix
+    # is.logical allowed because qr etc. treat logical vars as numeric
+    if(!is.numeric(x[,1]) && !is.logical(x[,1]))
+        stop("non-numeric column in 'x'")
+    # ensure all columns in x are named (needed for names in vcov etc.)
+    # use the same naming convention as lm (prefix for unnamed cols is "V")
+    missing.colnames <-
+        if(is.null(colnames(x))) 1:NCOL(x)
+        else                     nchar(colnames(x)) == 0
+    colnames(x)[missing.colnames] <-
+        c("(Intercept)",
+          paste("V", seq_len(NCOL(x) - 1), sep = ""))[missing.colnames]
+    duplicated <- which(duplicated(colnames(x)))
+    if(length(duplicated))
+        stop("column name \"", colnames(x)[duplicated[1]],
+             "\" in 'x' is duplicated")
+    x
+}
+check.linmod.y <- function(x, y)
+{
+    # as.vector(as.matrix(y)) is necessary when y is a data.frame
+    # (because as.vector alone on a data.frame returns a data.frame)
+    y <- as.vector(as.matrix(y))
+    if(length(y) == 0)
+        stop("'y' is empty")
+    if(anyNA(y))
+        stop("NA in 'y'")
+    if(!is.numeric(y) && !is.logical(y))
+        stop("'y' is not numeric or logical")
+    if(length(y) != nrow(x))
+        stop("nrow(x) is ", nrow(x), " but length(y) is ", length(y))
+    y
+}
+do.linmod.fit <- function(x, y)
+{
+    # workhorse function for fitting linear models
+    # essential processing and sanity checks on x and y are already completed
+    # x is a numeric matrix, y is a numeric vector
 
     qx <- qr(x)                            # QR-decomposition of x
     if(qx$rank < ncol(x))
@@ -40,108 +125,13 @@ linmod.fit <- function(x = stop("no 'x' argument"),
     names(fitted.values) <- rownames(x)
     colnames(coef) <- colnames(y)
 
-    fit <- list(coefficients  = coef,
-                residuals     = y - fitted.values,
-                fitted.values = fitted.values,
-                vcov          = vcov,
-                sigma         = sqrt(sigma2),
-                df.residual   = df.residual)
-
-    class(fit) <- "linmod"
-    fit
-}
-linmod.default <- function(x = stop("no 'x' argument"),
-                           y = stop("no 'y' argument"),
-                           keep = FALSE,
-                           ...)
-{
-    stop.if.dot.arg.used(...)
-    x.original <- x
-    xmat <- as.matrix(x)
-    # use name "(Intercept)" here so coef names match linmod.formula
-    x <- cbind("(Intercept)"=1, xmat)
-    fit <- linmod.fit(x, y)
-    fit$call <- match.call()
-    if(keep) {
-        fit$x <- xmat
-        # save y as a matrix so can use colname to remember response name
-        # (we already checked that it's safe to do this in check.linmod.y)
-        colname <- deparse(substitute(y))[1]
-        colname <- gsub(" ", "", substr(colname, 1, 100)) # strip spaces, truncate
-        fit$y <- as.matrix(y, ncol=1)
-        colnames(fit$y) <- colname
-    }
-    fit
-}
-linmod.formula <- function(formula = stop("no 'formula' argument"),
-                           data = parent.frame(),
-                           keep = FALSE,
-                           ...)
-{
-    stop.if.dot.arg.used(...)
-    # note that na.action=na.pass because we will catch NAs later
-    # in linmod.fit, for uniformity with linmod.default
-    mf <- model.frame(formula=formula, data=data, na.action=na.pass)
-    terms <- attr(mf, "terms")
-    x <- model.matrix(terms, mf)
-    y <- model.response(mf)
-    fit <- linmod.fit(x, y)
-    fit$terms <- terms
-    fit$xlevels <- .getXlevels(terms, mf) # will be used by predict.linmod
-    fit$call <- match.call()
-    if(keep) {
-        # TODO reconsider how factors in x should be handled here
-        varnames <- names(attr(terms, "dataClasses"))
-        int.col <- attr(terms, "intercept") # intercept column nbr in x
-        if(int.col != 0)                    # drop intercept column
-            x <- x[, -int.col, drop=FALSE]  # for compat with linmod.default
-        fit$x <- x # any factors will have been expanded by model.frame
-        # save y as a matrix so can use colname to remember response name
-        fit$y <- as.matrix(y, ncol=1)
-        colnames(fit$y) <- varnames[attr(terms, "response")]
-    }
-    fit
-}
-check.linmod.x <- function(x)
-{
-    if(!is.matrix(x))
-        stop("'x' is not a matrix or could not be converted to a matrix")
-    if(NROW(x) == 0 || NCOL(x) == 0)
-        stop("'x' is empty")
-    if(anyNA(x))
-        stop("NA in 'x'")
-    # checking just the first column of x suffices because x is a matrix
-    # is.logical allowed because qr etc. know how to deal with logical vars
-    if(!is.numeric(x[,1]) && !is.logical(x[,1]))
-        stop("non-numeric column in 'x'")
-    # ensure all columns in x are named (needed for names in vcov etc.)
-    # use the same naming convention as lm (prefix for unnamed cols is "V")
-    missing.colnames <-
-        if(is.null(colnames(x))) 1:NCOL(x)
-        else                     nchar(colnames(x)) == 0
-    colnames(x)[missing.colnames] <-
-        c("(Intercept)",
-          paste("V", seq_len(NCOL(x)-1), sep=""))[missing.colnames]
-    duplicated <- which(duplicated(colnames(x)))
-    if(length(duplicated))
-        stop("column name \"", colnames(x)[duplicated[1]],
-             "\" in 'x' is duplicated")
-    x
-}
-check.linmod.y <- function(x, y)
-{
-    # as.vector(as.matrix(y)) is necessary when y is a data.frame
-    # (as.vector alone on a data.frame returns a data.frame)
-    y <- as.vector(as.matrix(y))
-    if(length(y) == 0)
-        stop("'y' is empty")
-    if(anyNA(y))
-        stop("NA in 'y'")
-    if(!is.numeric(y) && !is.logical(y))
-        stop("'y' is not numeric or logical")
-    if(length(y) != nrow(x))
-        stop("nrow(x) is ", nrow(x), " but length(y) is ", length(y))
-    y
+    # returned fields match lm's fields
+    list(coefficients  = coef,
+         residuals     = y - fitted.values,
+         fitted.values = fitted.values,
+         vcov          = vcov,
+         sigma         = sqrt(sigma2),
+         df.residual   = df.residual)
 }
 predict.linmod <- function(object = stop("no 'object' argument"),
                            newdata = NULL,
@@ -150,37 +140,49 @@ predict.linmod <- function(object = stop("no 'object' argument"),
 {
     stopifnot(inherits(object, "linmod"))
     stop.if.dot.arg.used(...)
-    if(!identical(type, "response"))
-        stop("the 'type' argument is not yet supported")
+    match.arg(type, "response") # the type argument is not yet supported
     if(is.null(newdata))
         yhat <- fitted(object)
     else {
         if(NROW(newdata) == 0)
-            stop("'newdata' is empty")      # prevents obscure message later
-        if(is.null(object$terms)) {         # model built with linmod.default?
-            # columns in newdata must be in the same order as orig x
-            x <- as.matrix(newdata)         # allows newdata to be a data.frame
-            x <- cbind(1, x)                # intercept column
-        } else                              # model built with linmod.formula
-            x <- process.newdata(object, newdata)
-        casenames <- rownames(x)
-        # TODO The following tests suffice to catch all incorrect input,
-        # but aren't ideal in that they don't always direct you to the
-        # root cause of the problem.
-        if(ncol(x) != length(object$coefficients))
-            stop("ncol(newdata) is ", ncol(x)-1, " but should be ",
-                 length(object$coefficients)-1) # -1 for intercept
+            stop("'newdata' is empty")      # preempt obscure message later
+
+        x <- if(is.null(object$terms))      # model built with linmod.default?
+                process.default.newdata(object, newdata)
+            else                            # model built with linmod.formula
+                process.formula.newdata(object, newdata)
+
+        # The following tests suffice to catch all illegal input.  However
+        # they aren't ideal in that they don't always direct you to the root
+        # cause of the problem (i.e. the error messages aren't always optimal).
+        nvar <- length(object$coefficients) - 1 # nbr vars, -1 for intercept
+        if(ncol(x) - 1 != nvar)
+            stop("ncol(newdata) is ", ncol(x) - 1, " but should be ", nvar)
         if(anyNA(x))
             stop("NA in 'newdata'")
-        if(!is.numeric(x[,1]) && !is.logical(x[1,]))
+        if(!is.numeric(x[,1]) && !is.logical(x[,1]))
             stop("non-numeric column in 'newdata' (after processing)")
-        yhat <- as.vector(x %*% coef(object))
-        names(yhat) <- casenames
+
+        yhat <- as.vector(do.predict.linmod(object, x))
+        names(yhat) <- rownames(x)
     }
     yhat
 }
-process.newdata <- function(object, newdata) # for models built with linmod.formula
+process.default.newdata <- function(object, newdata)
 {
+    # process newdata for models built with linmod.default
+
+    x <- if(is.vector(newdata))     # allow newdata to be a vector
+            matrix(newdata, ncol = length(object$coefficients) - 1)
+         else
+            as.matrix(newdata)      # allow newdata to be a data.frame
+
+    cbind(1, x)                     # return data with an intercept column
+}
+process.formula.newdata <- function(object, newdata)
+{
+    # process newdata for models built with linmod.formula
+
     terms <- object$terms
     dataClasses <- attr(terms, "dataClasses")
     newdata <- as.data.frame(newdata) # allows newdata to be a matrix
@@ -188,17 +190,17 @@ process.newdata <- function(object, newdata) # for models built with linmod.form
     # The code below preempts code in model.frame that issues
     #   Warning: 'newdata' had M rows but variables found have N rows
     # This code gives a clearer error message.
-    # This check is necessary else model.frame can return bad data.
+    # Variable names check is necessary else model.frame can return bad data.
     varnames <- names(dataClasses)
     varnames <- varnames[-attr(terms, "response")]
     missing <- which(!(varnames %in% colnames(newdata)))
     if(length(missing))
-        stop("variable '", varnames[missing[1]],
-             "' used when building the model is not in colnames(newdata)")
+        stop("variable '", varnames[missing[1]], "' is missing from newdata")
 
     terms <- delete.response(terms)
     # na.action=na.pass because we will catch NAs after (for clearer error msg)
-    newdata <- model.frame(terms, newdata, na.action=na.pass, xlev=object$xlevels)
+    newdata <- model.frame(terms, newdata, na.action = na.pass,
+                           xlev = object$xlevels)
     if(anyNA(newdata))
         stop("NA in 'newdata'")
     if(NROW(newdata) != NROW(newdata))    # paranoia, shouldn't be needed
@@ -207,6 +209,14 @@ process.newdata <- function(object, newdata) # for models built with linmod.form
     .checkMFClasses(dataClasses, newdata) # check types in newdata match original data
     model.matrix(terms, newdata)
 }
+do.predict.linmod <- function(object, x)
+{
+    # workhorse function for linear model predictions
+    # processing by model.matrix etc. and sanity checks on x already completed
+    # x is a numeric matrix (if model has intercept, first col of x is all 1s)
+
+    x %*% coef(object)
+}
 summary.linmod <- function(object = stop("no 'object' argument"), ...)
 {
     stop.if.dot.arg.used(...)
@@ -216,12 +226,13 @@ summary.linmod <- function(object = stop("no 'object' argument"), ...)
     coefficients <- cbind(Estimate = coef(object),
                           StdErr   = se,
                           t.value  = t.value,
-                          p.value  = 2 * pt(-abs(t.value), df=object$df))
+                          p.value  = 2 * pt(-abs(t.value), df = object$df))
 
     retval <- list(call         = object$call,
                    coefficients = coefficients)
 
     class(retval) <- "summary.linmod"
+
     retval
 }
 print.linmod <- function(x = stop("no 'x' argument"), ...)
@@ -235,16 +246,16 @@ print.summary.linmod <- function(x = stop("no 'x' argument"), ...)
 {
     stop.if.dot.arg.used(...)
     print.model.call(x)
-    printCoefmat(x$coefficients, signif.stars=FALSE,
-                 P.value=TRUE, has.Pvalue=TRUE)
+    printCoefmat(x$coefficients, signif.stars = FALSE,
+                 P.value = TRUE, has.Pvalue = TRUE)
     invisible(x)
 }
 print.model.call <- function(x)
 {
-    cat("Call: ") # lm has a newline here, but a space is more compact
+    cat("Call: ") # print.lm has a newline here, but a space is more compact
     # use paste0 to convert vector of strings to single string if necessary
-    cat(strwrap(paste0(deparse(x$call, control=NULL, nlines=5),
-                       sep=" ", collapse=" "), exdent=6), sep="\n")
+    cat(strwrap(paste0(deparse(x$call, control = NULL, nlines = 5),
+                       sep = " ", collapse = " "), exdent = 6), sep = "\n")
     cat("\n")
 }
 # stop.if.dot.arg.used will cause an error message if any args are passed to it.
